@@ -1,12 +1,12 @@
-.embedded_sf_method = \(data,target,E = 3,tau = 1,nb = NULL,detrend = FALSE){
+.embedded_sf_method = \(data,target,E = 3,tau = 1,style = 1,nb = NULL,detrend = FALSE){
   vec = .uni_lattice(data,target,detrend)
   if (is.null(nb)) nb = .internal_lattice_nb(data)
-  return(RcppGenLatticeEmbeddings(vec,nb,E,tau))
+  return(RcppGenLatticeEmbeddings(vec,nb,E,tau,style))
 }
 
-.embedded_spatraster_method = \(data,target,E = 3,tau = 1,detrend = FALSE){
+.embedded_spatraster_method = \(data,target,E = 3,tau = 1,style = 1,detrend = FALSE){
   mat = .uni_grid(data,target,detrend)
-  return(RcppGenGridEmbeddings(mat,E,tau))
+  return(RcppGenGridEmbeddings(mat,E,tau,style))
 }
 
 #' embedding spatial cross sectional data
@@ -15,6 +15,7 @@
 #' @param target name of target variable.
 #' @param E (optional) embedding dimensions.
 #' @param tau (optional) step of spatial lags.
+#' @param style (optional) embedding style (`0` includes current state, `1` excludes it).
 #' @param nb (optional) neighbours list.
 #' @param detrend (optional) whether to remove the linear trend.
 #'
