@@ -1,22 +1,24 @@
-.fnn_sf_method = \(data, target, lib = NULL, pred = NULL, E = 1:10, tau = 1, nb = NULL,
-                   rt = 10, eps = 2, threads = detectThreads(), detrend = TRUE){
+.fnn_sf_method = \(data, target, lib = NULL, pred = NULL, E = 1:10, tau = 1, style = 1, dist.metric = "L1",
+                   nb = NULL, rt = 10, eps = 2, threads = detectThreads(), detrend = TRUE){
   vec = .uni_lattice(data,target,detrend)
   rt = .check_inputelementnum(rt,max(E))
   eps = .check_inputelementnum(eps,max(E))
   if (is.null(lib)) lib = which(!is.na(vec))
   if (is.null(pred)) pred = lib
   if (is.null(nb)) nb = .internal_lattice_nb(data)
-  return(RcppFNN4Lattice(vec,nb,rt,eps,lib,pred,E,tau,threads))
+  return(RcppFNN4Lattice(vec, nb, rt, eps, lib, pred, E, tau, style,
+                         .check_distmetric(dist.metric),threads))
 }
 
-.fnn_spatraster_method = \(data, target, lib = NULL, pred = NULL, E = 1:10, tau = 1,
-                           rt = 10, eps = 2, threads = detectThreads(), detrend = TRUE){
+.fnn_spatraster_method = \(data, target, lib = NULL, pred = NULL, E = 1:10, tau = 1, style = 1,
+                           dist.metric = "L1", rt = 10, eps = 2, threads = detectThreads(), detrend = TRUE){
   mat = .uni_grid(data,target,detrend)
   rt = .check_inputelementnum(rt,max(E))
   eps = .check_inputelementnum(eps,max(E))
   if (is.null(lib)) lib = which(!is.na(mat), arr.ind = TRUE)
   if (is.null(pred)) pred = lib
-  return(RcppFNN4Grid(mat,rt,eps,lib,pred,E,tau,threads))
+  return(RcppFNN4Grid(mat,rt,eps,lib, pred, E, tau, style,
+                      .check_distmetric(dist.metric),threads))
 }
 
 #' false nearest neighbours
@@ -24,6 +26,7 @@
 #' @inheritParams embedded
 #' @param lib (optional) libraries indices.
 #' @param pred (optional) predictions indices.
+#' @param dist.metric (optional) distance metric (`L1`: Manhattan, `L2`: Euclidean).
 #' @param rt (optional) escape factor.
 #' @param eps (optional) neighborhood diameter.
 #' @param threads (optional) number of threads to use.
