@@ -1,5 +1,5 @@
 .internal_xmapdf_print = \(x,keyname = "libsizes",significant = FALSE){
-  resdf = x$xmap
+  resdf = x[[1]]
   bidirectional = x$bidirectional
   if (bidirectional){
     if (significant) {
@@ -87,20 +87,31 @@ print.sc_res = \(x,...){
 #' plot ccm result
 #' @noRd
 #' @export
-plot.ccm_res = \(x, family = "serif", legend_texts = NULL,
+plot.ccm_res = \(x, family = "serif",
+                 legend_sig = TRUE, legend_texts = NULL,
                  legend_cols = c("#ed795b","#608dbe"),
                  draw_ci = FALSE, ci_alpha = 0.25,
                  xbreaks = NULL, xlimits = NULL,
                  ybreaks = seq(0, 1, by = 0.1),
                  ylimits = c(-0.05, 1),
                  ylabel = expression(rho), ...){
-  resdf = x$xmap
+  resdf = x[[1]]
   bidirectional = x$bidirectional
 
   if(is.null(xbreaks)) xbreaks = resdf$libsizes
   if(is.null(xlimits)) xlimits = c(min(xbreaks)-1,max(xbreaks)+1)
-  if (is.null(legend_texts)) legend_texts = c(paste0(x$varname[2], " xmap ", x$varname[1]),
-                                              paste0(x$varname[1], " xmap ", x$varname[2]))
+  if (is.null(legend_texts)){
+    legend_texts = c(paste0(x$varname[2], " xmap ", x$varname[1]),
+                     paste0(x$varname[1], " xmap ", x$varname[2]))
+    if (legend_sig){
+      pval = resdf |>
+        dplyr::slice_tail(n = 1) |>
+        dplyr::select(y_xmap_x_sig,x_xmap_y_sig) |>
+        unlist() |>
+        round(3)
+      legend_texts = paste0(legend_texts,", P = ",pval)
+    }
+  }
   legend_texts = .check_inputelementnum(legend_texts,2)
   legend_cols = .check_inputelementnum(legend_cols,2)
   names(legend_cols) = c("x - y","y - x")
@@ -158,10 +169,9 @@ plot.ccm_res = \(x, family = "serif", legend_texts = NULL,
 #' @export
 plot.cmc_res = \(x, ...){
   xmap = x[-1]
-  names(xmap) = c("xmap", "varname", "bidirectional")
   class(xmap) = "ccm"
-  draw_ci = FALSE
-  fig1 = plot.ccm_res(xmap,draw_ci = draw_ci,ylabel = "Causal Score",...)
+  fig1 = plot.ccm_res(xmap,legend_sig = FALSE,draw_ci = FALSE,
+                      ylabel = "Causal Score",...)
   return(fig1)
 }
 
