@@ -1,8 +1,9 @@
-.gcmc_sf_method = \(data, cause, effect, libsizes = NULL, E = 3, tau = 1, k = pmin(E^2), lib = NULL, pred = NULL, nb = NULL,
-                    threads = detectThreads(), parallel.level = "low", bidirectional = TRUE, detrend = FALSE, progressbar = TRUE){
+.gcmc_sf_method = \(data, cause, effect, libsizes = NULL, E = 3, tau = 1, k = pmin(E^2), lib = NULL, pred = NULL, style = 1, dist.metric = "L2",
+                    nb = NULL, threads = detectThreads(), parallel.level = "low", bidirectional = TRUE, detrend = FALSE, progressbar = TRUE){
   varname = .check_character(cause, effect)
   E = .check_inputelementnum(E,2)
   tau = .check_inputelementnum(tau,2)
+  k = .check_inputelementnum(k,2)
   pl = .check_parallellevel(parallel.level)
   .varname = .internal_varname()
   if (is.null(nb)) nb = .internal_lattice_nb(data)
@@ -23,18 +24,21 @@
 
   x_xmap_y = NULL
   if (bidirectional){
-    x_xmap_y = RcppGCMC4Lattice(cause,effect,nb,libsizes,lib,pred,E,tau,k,0,threads,pl,progressbar)
+    x_xmap_y = RcppGCMC4Lattice(cause,effect,nb,libsizes,lib,pred,E,tau,k[1],0,style,
+                                .check_distmetric(dist.metric),threads,pl,progressbar)
   }
-  y_xmap_x = RcppGCMC4Lattice(effect,cause,nb,libsizes,lib,pred,rev(E),rev(tau),k,0,threads,pl,progressbar)
+  y_xmap_x = RcppGCMC4Lattice(effect,cause,nb,libsizes,lib,pred,rev(E),rev(tau),k[2],0,
+                              style, .check_distmetric(dist.metric),threads,pl,progressbar)
 
   return(.bind_intersectdf(varname,x_xmap_y,y_xmap_x,bidirectional))
 }
 
-.gcmc_spatraster_method = \(data, cause, effect, libsizes = NULL, E = 3, tau = 1, k = pmin(E^2), lib = NULL, pred = NULL,
+.gcmc_spatraster_method = \(data, cause, effect, libsizes = NULL, E = 3, tau = 1, k = pmin(E^2), lib = NULL, pred = NULL, style = 1, dist.metric = "L2",
                             threads = detectThreads(), parallel.level = "low", bidirectional = TRUE, detrend = FALSE, progressbar = TRUE){
   varname = .check_character(cause, effect)
   E = .check_inputelementnum(E,2)
   tau = .check_inputelementnum(tau,2)
+  k = .check_inputelementnum(k,2)
   pl = .check_parallellevel(parallel.level)
   .varname = .internal_varname()
   data = data[[varname]]
@@ -53,9 +57,11 @@
 
   x_xmap_y = NULL
   if (bidirectional){
-    x_xmap_y = RcppGCMC4Grid(causemat,effectmat,libsizes,lib,pred,E,tau,k,0,threads,pl,progressbar)
+    x_xmap_y = RcppGCMC4Grid(causemat,effectmat,libsizes,lib,pred,E,tau,k[1],0,style,
+                             .check_distmetric(dist.metric),threads,pl,progressbar)
   }
-  y_xmap_x = RcppGCMC4Grid(effectmat,causemat,libsizes,lib,pred,rev(E),rev(tau),k,0,threads,pl,progressbar)
+  y_xmap_x = RcppGCMC4Grid(effectmat,causemat,libsizes,lib,pred,rev(E),rev(tau),k[2],0,
+                           style,.check_distmetric(dist.metric),threads,pl,progressbar)
 
   return(.bind_intersectdf(varname,x_xmap_y,y_xmap_x,bidirectional))
 }
@@ -71,6 +77,8 @@
 #' @param k (optional) number of nearest neighbors.
 #' @param lib (optional) libraries indices.
 #' @param pred (optional) predictions indices.
+#' @param style (optional) embedding style (`0` includes current state, `1` excludes it).
+#' @param dist.metric (optional) distance metric (`L1`: Manhattan, `L2`: Euclidean).
 #' @param nb (optional) neighbours list.
 #' @param threads (optional) number of threads to use.
 #' @param parallel.level (optional) level of parallelism, `low` or `high`.
@@ -92,7 +100,7 @@
 #' @examples
 #' columbus = sf::read_sf(system.file("case/columbus.gpkg", package="spEDM"))
 #' \donttest{
-#' g = gcmc(columbus,"hoval","crime",E = 2,k = 25)
+#' g = gcmc(columbus,"hoval","crime",E = 7,k = 18)
 #' g
 #' }
 methods::setMethod("gcmc", "sf", .gcmc_sf_method)
