@@ -98,23 +98,27 @@ std::vector<std::vector<double>> CppLaggedVal4Grid(
  * a different lagged value or the original element.
  *
  * Parameters:
- *   mat  - A 2D vector representing the grid data.
- *   E    - The number of embedding dimensions (columns in the resulting matrix).
- *   tau  - The spatial lag step for constructing lagged state-space vectors.
+ *   mat   - A 2D vector representing the grid data.
+ *   E     - The number of embedding dimensions (columns in the resulting matrix).
+ *   tau   - The spatial lag step for constructing lagged state-space vectors.
+ *   style - Embedding style selector:
+ *             - style = 0: embedding includes current state as the first dimension.
+ *             - style != 0: embedding excludes current state.
  *
  * Returns:
  *   A 2D vector (matrix) where each row contains the averaged lagged variables for
  *   each embedding dimension (column). Columns where all values are NaN are removed.
  *
  * Note:
- *   When tau = 0, the lagged variables are calculated for lag steps of 0, 1, ..., E-1.
- *   When tau > 0, the lagged variables are calculated for lag steps of tau, 2*tau, ..., E*tau,
- *   and this means the actual lag steps form an arithmetic sequence with a common difference of tau.
+ *   When tau = 0, lagged variables are calculated for lag steps 0, 1, ..., E-1.
+ *   When tau > 0 and style = 0, lagged variables are calculated for lag steps 0, tau, 2*tau, ..., (E-1)*tau.
+ *   When tau > 0 and style != 0, lagged variables are calculated for lag steps tau, 2*tau, ..., E*tau.
  */
 std::vector<std::vector<double>> GenGridEmbeddings(
     const std::vector<std::vector<double>>& mat,
     int E,
-    int tau
+    int tau,
+    int style = 1
 );
 
 /**
