@@ -22,8 +22,11 @@
  *   - pred_indices: A vector of indices indicating the prediction set.
  *   - E: A vector of embedding dimensions to evaluate.
  *   - b: A vector of nearest neighbors to use for prediction.
- *   - tau: The spatial lag step for constructing lagged state-space vectors.
- *   - threads: Number of threads used from the global pool.
+ *   - tau: The spatial lag step for constructing lagged state-space vectors. Default is 1.
+ *   - style: Embedding style selector (0: includes current state, 1: excludes it).  Default is 1 (excludes current state).
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
+ *   - threads: Number of threads used from the global pool. Default is 8.
  *
  * Returns:
  *   A 2D vector where each row contains [E, b, rho, mae, rmse] for a given embedding dimension.
@@ -34,8 +37,11 @@ std::vector<std::vector<double>> Simplex4Grid(const std::vector<std::vector<doub
                                               const std::vector<int>& pred_indices,
                                               const std::vector<int>& E,
                                               const std::vector<int>& b,
-                                              int tau,
-                                              int threads);
+                                              int tau = 1,
+                                              int style = 1,
+                                              int dist_metric = 2,
+                                              bool dist_average = true,
+                                              int threads = 8);
 
 /*
  * Evaluates prediction performance of different theta parameters for grid data using the S-mapping method.
@@ -46,10 +52,13 @@ std::vector<std::vector<double>> Simplex4Grid(const std::vector<std::vector<doub
  *   - lib_indices: A vector of indices indicating the library (training) set.
  *   - pred_indices: A vector of indices indicating the prediction set.
  *   - theta: A vector of weighting parameters for distance calculation in SMap.
- *   - E: The embedding dimension to evaluate.
- *   - tau: The spatial lag step for constructing lagged state-space vectors.
- *   - b: Number of nearest neighbors to use for prediction.
- *   - threads: Number of threads used from the global pool.
+ *   - E: The embedding dimension to evaluate. Default is 3.
+ *   - tau: The spatial lag step for constructing lagged state-space vectors. Default is 1.
+ *   - b: Number of nearest neighbors to use for prediction. Default is 4.
+ *   - style: Embedding style selector (0: includes current state, 1: excludes it).  Default is 1 (excludes current state).
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
+ *   - threads: Number of threads used from the global pool. Default is 8.
  *
  * Returns:
  *   A 2D vector where each row contains [theta, rho, mae, rmse] for a given theta value.
@@ -59,13 +68,16 @@ std::vector<std::vector<double>> SMap4Grid(const std::vector<std::vector<double>
                                            const std::vector<int>& lib_indices,
                                            const std::vector<int>& pred_indices,
                                            const std::vector<double>& theta,
-                                           int E,
-                                           int tau,
-                                           int b,
-                                           int threads);
+                                           int E = 3,
+                                           int tau = 1,
+                                           int b = 4,
+                                           int style = 1,
+                                           int dist_metric = 2,
+                                           bool dist_average = true,
+                                           int threads = 8);
 
 /**
- * @brief Evaluate intersection cardinality (IC) for spatial grid embeddings.
+ * @brief Evaluate intersection cardinality (IC) for spatial grid data.
  *
  * This function computes the intersection cardinality between the k-nearest neighbors
  * of grid-embedded source and target spatial variables, across a range of embedding dimensions (E)
@@ -87,6 +99,8 @@ std::vector<std::vector<double>> SMap4Grid(const std::vector<std::vector<double>
  * @param b Vector of neighbor counts (k) used to compute IC.
  * @param tau Spatial embedding spacing (lag). Determines distance between embedding neighbors.
  * @param exclude Number of nearest neighbors to exclude in IC computation.
+ * @param style Embedding style selector (0: includes current state, 1: excludes it). 
+ * @param dist_metric Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param threads Maximum number of threads to use.
  * @param parallel_level If > 0, enables parallel evaluation of b for each E.
  *
@@ -98,9 +112,11 @@ std::vector<std::vector<double>> IC4Grid(const std::vector<std::vector<double>>&
                                          const std::vector<size_t>& pred_indices,
                                          const std::vector<int>& E,
                                          const std::vector<int>& b,
-                                         int tau,
-                                         int exclude,
-                                         int threads,
-                                         int parallel_level);
+                                         int tau = 1,
+                                         int exclude = 0,
+                                         int style = 1,
+                                         int dist_metric = 2,
+                                         int threads = 8,
+                                         int parallel_level = 0);
 
 #endif // Forecast4Grid_H
