@@ -23,8 +23,11 @@
  *   - pred_indices: A vector of indices indicating the prediction set.
  *   - E: A vector of embedding dimensions to evaluate.
  *   - b: A vector of nearest neighbor values to evaluate.
- *   - tau: The spatial lag step for constructing lagged state-space vectors.
- *   - threads: Number of threads used from the global pool.
+ *   - tau: The spatial lag step for constructing lagged state-space vectors. Default is 1.
+ *   - style: Embedding style selector (0: includes current state, 1: excludes it).  Default is 1 (excludes current state).
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
+ *   - threads: Number of threads used from the global pool. Default is 8.
  *
  * Returns:
  *   A 2D vector where each row contains [E, b, rho, mae, rmse] for a given combination of E and b.
@@ -36,8 +39,11 @@ std::vector<std::vector<double>> Simplex4Lattice(const std::vector<double>& sour
                                                  const std::vector<int>& pred_indices,
                                                  const std::vector<int>& E,
                                                  const std::vector<int>& b,
-                                                 int tau,
-                                                 int threads);
+                                                 int tau = 1,
+                                                 int style = 1,
+                                                 int dist_metric = 2,
+                                                 bool dist_average = true,
+                                                 int threads = 8);
 
 /*
  * Evaluates prediction performance of different theta parameters for lattice data using the s-mapping method.
@@ -49,10 +55,13 @@ std::vector<std::vector<double>> Simplex4Lattice(const std::vector<double>& sour
  *   - lib_indices: A vector of indices indicating the library (training) set.
  *   - pred_indices: A vector of indices indicating the prediction set.
  *   - theta: A vector of weighting parameters for distance calculation in SMap.
- *   - E: The embedding dimension to evaluate.
- *   - tau: The spatial lag step for constructing lagged state-space vectors.
- *   - b: Number of nearest neighbors to use for prediction.
- *   - threads: Number of threads used from the global pool.
+ *   - E: The embedding dimension to evaluate. Default is 3.
+ *   - tau: The spatial lag step for constructing lagged state-space vectors. Default is 1.
+ *   - b: Number of nearest neighbors to use for prediction. Default is 4.
+ *   - style: Embedding style selector (0: includes current state, 1: excludes it).  Default is 1 (excludes current state).
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
+ *   - threads: Number of threads used from the global pool. Default is 8.
  *
  * Returns:
  *   A 2D vector where each row contains [theta, rho, mae, rmse] for a given theta value.
@@ -63,13 +72,16 @@ std::vector<std::vector<double>> SMap4Lattice(const std::vector<double>& source,
                                               const std::vector<int>& lib_indices,
                                               const std::vector<int>& pred_indices,
                                               const std::vector<double>& theta,
-                                              int E,
-                                              int tau,
-                                              int b,
-                                              int threads);
+                                              int E = 3,
+                                              int tau = 1,
+                                              int b = 4,
+                                              int style = 1,
+                                              int dist_metric = 2,
+                                              bool dist_average = true,
+                                              int threads = 8);
 
 /**
- * Compute Intersection Cardinality AUC over Lattice Embedding Settings.
+ * Compute Intersection Cardinality AUC over spatial lattice data.
  *
  * This function computes the causal strength between two lattice-structured time series
  * (`source` and `target`) by evaluating the Intersection Cardinality (IC) curve, and
@@ -91,11 +103,13 @@ std::vector<std::vector<double>> SMap4Lattice(const std::vector<double>& source,
  * @param b              Vector of neighbor sizes to try.
  * @param tau            Embedding delay (usually 1 for lattice).
  * @param exclude        Number of nearest neighbors to exclude (e.g., temporal or spatial proximity).
+ * @param style          Embedding style selector (0: includes current state, 1: excludes it). 
+ * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param threads        Number of threads for parallel computation.
  * @param parallel_level Flag indicating whether to use multi-threading (0: serial, 1: parallel).
  *
  * @return A vector of size `E.size() * b.size()`, each element is a vector:
- *         [embedding_dimension, neighbor_size, auc_value].
+ *         [embedding_dimension, neighbor_size, auc_value, p value].
  *         If inputs are invalid or no prediction point is valid, the AUC value is NaN.
  *
  * @note
@@ -110,9 +124,11 @@ std::vector<std::vector<double>> IC4Lattice(const std::vector<double>& source,
                                             const std::vector<size_t>& pred_indices,
                                             const std::vector<int>& E,
                                             const std::vector<int>& b,
-                                            int tau,
-                                            int exclude,
-                                            int threads,
-                                            int parallel_level);
+                                            int tau = 1,
+                                            int exclude = 0,
+                                            int style = 1,
+                                            int dist_metric = 2,
+                                            int threads = 8,
+                                            int parallel_level = 0);
 
 #endif // Forecast4Lattice_H
