@@ -32,6 +32,8 @@
  * @param threads              The number of threads to use for parallel processing.
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * @param row_size_mark        If true, use the row-wise libsize to mark the libsize; if false, use col-wise libsize.
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average         Whether to average distance by the number of valid vector components.
  *
  * @return  A vector of pairs, where each pair contains the library size and the corresponding cross mapping result.
  */
@@ -48,7 +50,10 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
     double theta,
     size_t threads,
     int parallel_level,
-    bool row_size_mark) {
+    bool row_size_mark,
+    int dist_metric,
+    bool dist_average
+) {
   // Extract row-wise and column-wise library sizes
   const int lib_size_row = lib_sizes[0];
   const int lib_size_col = lib_sizes[1];
@@ -91,9 +96,9 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
     double rho = std::numeric_limits<double>::quiet_NaN();
     // Run cross map and store results
     if (simplex) {
-      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b);
+      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b, dist_metric, dist_average);
     } else {
-      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta);
+      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta, dist_metric, dist_average);
     }
     x_xmap_y[idx] = std::make_pair(libsize, rho);
   };
@@ -128,6 +133,8 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
  * @param theta                Distance weighting parameter for S-mapping
  * @param threads              The number of threads to use for parallel processing
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average         Whether to average distance by the number of valid vector components.
  *
  * @return A vector of pairs, where each pair contains the library size and the corresponding cross mapping result.
  */
@@ -143,7 +150,10 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
     bool simplex,
     double theta,
     size_t threads,
-    int parallel_level) {
+    int parallel_level,
+    int dist_metric,
+    bool dist_average
+) {
   int max_lib_size = lib_indices.size();
 
   // No possible library variation if using all vectors
@@ -153,9 +163,9 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
     double rho = std::numeric_limits<double>::quiet_NaN();
     // Run cross map and store results
     if (simplex) {
-      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b);
+      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b, dist_metric, dist_average);
     } else {
-      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta);
+      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta, dist_metric, dist_average);
     }
     x_xmap_y.emplace_back(lib_size, rho);
     return x_xmap_y;
@@ -189,9 +199,9 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
       double rho = std::numeric_limits<double>::quiet_NaN();
       // Run cross map and store results
       if (simplex) {
-        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b);
+        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, dist_metric, dist_average);
       } else {
-        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta);
+        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta, dist_metric, dist_average);
       }
 
       std::pair<int, double> result(lib_size, rho); // Store the product of row and column library sizes
@@ -229,9 +239,9 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
       double rho = std::numeric_limits<double>::quiet_NaN();
       // Run cross map and store results
       if (simplex) {
-        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b);
+        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, dist_metric, dist_average);
       } else {
-        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta);
+        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta, dist_metric, dist_average);
       }
 
       std::pair<int, double> result(lib_size, rho); // Store the product of row and column library sizes
@@ -260,6 +270,10 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
  * @param theta          The distance weighting parameter for S-Mapping (ignored if simplex is true).
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
+ * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average   Whether to average distance by the number of valid vector components.
+ * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -278,6 +292,10 @@ std::vector<std::vector<double>> GCCM4Grid(
     double theta,
     int threads,
     int parallel_level,
+    int style,
+    int dist_metric,
+    bool dist_average,
+    bool single_sig,
     bool progressbar
 ) {
   // If b is not provided correctly, default it to E + 2
@@ -300,7 +318,7 @@ std::vector<std::vector<double>> GCCM4Grid(
   }
 
   // Generate embeddings for xMatrix
-  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau);
+  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau, style);
 
   // Ensure the maximum value does not exceed totalRow or totalCol
   int max_lib_size_row = totalRow;
@@ -394,7 +412,9 @@ std::vector<std::vector<double>> GCCM4Grid(
           theta,
           threads_sizet,
           parallel_level,
-          row_size_mark);
+          row_size_mark,
+          dist_metric,
+          dist_average);
         bar++;
       }
     } else {
@@ -414,7 +434,9 @@ std::vector<std::vector<double>> GCCM4Grid(
           theta,
           threads_sizet,
           parallel_level,
-          row_size_mark);
+          row_size_mark,
+          dist_metric,
+          dist_average);
       }
     }
   } else {
@@ -437,7 +459,9 @@ std::vector<std::vector<double>> GCCM4Grid(
           theta,
           threads_sizet,
           parallel_level,
-          row_size_mark);
+          row_size_mark,
+          dist_metric,
+          dist_average);
         bar++;
       }, threads_sizet);
     } else {
@@ -457,7 +481,9 @@ std::vector<std::vector<double>> GCCM4Grid(
           theta,
           threads_sizet,
           parallel_level,
-          row_size_mark);
+          row_size_mark,
+          dist_metric,
+          dist_average);
       }, threads_sizet);
     }
   }
@@ -476,22 +502,47 @@ std::vector<std::vector<double>> GCCM4Grid(
     grouped_results[result.first].push_back(result.second);
   }
 
-  std::vector<std::vector<double>> final_results;
-  for (const auto& group : grouped_results) {
-    double mean_value = CppMean(group.second, true);
-    final_results.push_back({static_cast<double>(group.first), mean_value});
-  }
-
   size_t n = pred.size();
-  // Calculate significance and confidence interval for each result
-  for (size_t i = 0; i < final_results.size(); ++i) {
-    double rho = final_results[i][1];
-    double significance = CppCorSignificance(rho, n);
-    std::vector<double> confidence_interval = CppCorConfidence(rho, n);
+  std::vector<std::vector<double>> final_results;
 
-    final_results[i].push_back(significance);
-    final_results[i].push_back(confidence_interval[0]);
-    final_results[i].push_back(confidence_interval[1]);
+  if (single_sig) {
+    // Calculate significance and confidence intervals using the mean of rho vector only.
+    for (const auto& group : grouped_results) {
+      double mean_value = CppMean(group.second, true);
+      final_results.push_back({static_cast<double>(group.first), mean_value});
+    }
+
+    // Calculate significance and confidence interval for each result
+    for (size_t i = 0; i < final_results.size(); ++i) {
+      double rho = final_results[i][1];
+      double significance = CppCorSignificance(rho, n);
+      std::vector<double> confidence_interval = CppCorConfidence(rho, n);
+
+      final_results[i].push_back(significance);
+      final_results[i].push_back(confidence_interval[0]);
+      final_results[i].push_back(confidence_interval[1]);
+    }
+  } else {
+    // Compute significance and confidence intervals directly from grouped correlation vectors
+    for (const auto& group : grouped_results) {
+      // Calculate the mean correlation coefficient from the group
+      double mean_value = CppMean(group.second, true);
+
+      // Compute significance (p-value) using the vector of correlations directly
+      double significance = CppMeanCorSignificance(group.second, n);
+
+      // Compute confidence interval using the vector of correlations directly
+      std::vector<double> confidence_interval = CppMeanCorConfidence(group.second, n);
+
+      // Store results: group ID, mean correlation, p-value, lower CI, upper CI
+      final_results.push_back({
+        static_cast<double>(group.first),
+        mean_value,
+        significance,
+        confidence_interval[0],
+        confidence_interval[1]
+      });
+    }
   }
 
   return final_results;
@@ -515,6 +566,10 @@ std::vector<std::vector<double>> GCCM4Grid(
  * @param theta          The distance weighting parameter for S-Mapping (ignored if simplex is true).
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
+ * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average   Whether to average distance by the number of valid vector components.
+ * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -533,6 +588,10 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
     double theta,
     int threads,
     int parallel_level,
+    int style,
+    int dist_metric,
+    bool dist_average,
+    bool single_sig,
     bool progressbar
 ) {
   // If b is not provided correctly, default it to E + 2
@@ -555,7 +614,7 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
   }
 
   // Generate embeddings for xMatrix
-  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau);
+  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau, style);
 
   int max_lib_size = static_cast<int>(lib.size()); // Maximum lib size
 
@@ -593,7 +652,9 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
           simplex,
           theta,
           threads_sizet,
-          parallel_level
+          parallel_level,
+          dist_metric,
+          dist_average
         );
         bar++;
       }
@@ -611,7 +672,9 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
           simplex,
           theta,
           threads_sizet,
-          parallel_level
+          parallel_level,
+          dist_metric,
+          dist_average
         );
       }
     }
@@ -633,7 +696,9 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
           simplex,
           theta,
           threads_sizet,
-          parallel_level
+          parallel_level,
+          dist_metric,
+          dist_average
         );
         bar++;
       }, threads_sizet);
@@ -652,7 +717,9 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
           simplex,
           theta,
           threads_sizet,
-          parallel_level
+          parallel_level,
+          dist_metric,
+          dist_average
         );
       }, threads_sizet);
     }
@@ -672,22 +739,47 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
     grouped_results[result.first].push_back(result.second);
   }
 
-  std::vector<std::vector<double>> final_results;
-  for (const auto& group : grouped_results) {
-    double mean_value = CppMean(group.second, true);
-    final_results.push_back({static_cast<double>(group.first), mean_value});
-  }
-
   size_t n = pred.size();
-  // Calculate significance and confidence interval for each result
-  for (size_t i = 0; i < final_results.size(); ++i) {
-    double rho = final_results[i][1];
-    double significance = CppCorSignificance(rho, n);
-    std::vector<double> confidence_interval = CppCorConfidence(rho, n);
+  std::vector<std::vector<double>> final_results;
 
-    final_results[i].push_back(significance);
-    final_results[i].push_back(confidence_interval[0]);
-    final_results[i].push_back(confidence_interval[1]);
+  if (single_sig) {
+    // Calculate significance and confidence intervals using the mean of rho vector only.
+    for (const auto& group : grouped_results) {
+      double mean_value = CppMean(group.second, true);
+      final_results.push_back({static_cast<double>(group.first), mean_value});
+    }
+
+    // Calculate significance and confidence interval for each result
+    for (size_t i = 0; i < final_results.size(); ++i) {
+      double rho = final_results[i][1];
+      double significance = CppCorSignificance(rho, n);
+      std::vector<double> confidence_interval = CppCorConfidence(rho, n);
+
+      final_results[i].push_back(significance);
+      final_results[i].push_back(confidence_interval[0]);
+      final_results[i].push_back(confidence_interval[1]);
+    }
+  } else {
+    // Compute significance and confidence intervals directly from grouped correlation vectors
+    for (const auto& group : grouped_results) {
+      // Calculate the mean correlation coefficient from the group
+      double mean_value = CppMean(group.second, true);
+
+      // Compute significance (p-value) using the vector of correlations directly
+      double significance = CppMeanCorSignificance(group.second, n);
+
+      // Compute confidence interval using the vector of correlations directly
+      std::vector<double> confidence_interval = CppMeanCorConfidence(group.second, n);
+
+      // Store results: group ID, mean correlation, p-value, lower CI, upper CI
+      final_results.push_back({
+        static_cast<double>(group.first),
+        mean_value,
+        significance,
+        confidence_interval[0],
+        confidence_interval[1]
+      });
+    }
   }
 
   return final_results;
