@@ -33,6 +33,8 @@
  * @param threads              The number of threads to use for parallel processing.
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * @param row_size_mark        If true, use the row-wise libsize to mark the libsize; if false, use col-wise libsize.
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average         Whether to average distance by the number of valid vector components.
  *
  * @return  A vector of pairs, where each pair contains the library size and the corresponding cross mapping result.
  */
@@ -49,7 +51,9 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
     double theta,
     size_t threads,
     int parallel_level,
-    bool row_size_mark
+    bool row_size_mark,
+    int dist_metric,
+    bool dist_average
 );
 
 /**
@@ -70,6 +74,8 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
  * @param theta                Distance weighting parameter for S-mapping
  * @param threads              The number of threads to use for parallel processing
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average         Whether to average distance by the number of valid vector components.
  *
  * @return A vector of pairs, where each pair contains the library size and the corresponding cross mapping result.
  */
@@ -85,7 +91,9 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
     bool simplex,
     double theta,
     size_t threads,
-    int parallel_level
+    int parallel_level,
+    int dist_metric,
+    bool dist_average
 );
 
 /**
@@ -106,6 +114,10 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
  * @param theta          The distance weighting parameter for S-Mapping (ignored if simplex is true).
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
+ * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average   Whether to average distance by the number of valid vector components.
+ * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -124,6 +136,10 @@ std::vector<std::vector<double>> GCCM4Grid(
     double theta,
     int threads,
     int parallel_level,
+    int style,
+    int dist_metric,
+    bool dist_average,
+    bool single_sig,
     bool progressbar
 );
 
@@ -145,6 +161,10 @@ std::vector<std::vector<double>> GCCM4Grid(
  * @param theta          The distance weighting parameter for S-Mapping (ignored if simplex is true).
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
+ * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average   Whether to average distance by the number of valid vector components.
+ * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -163,6 +183,10 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
     double theta,
     int threads,
     int parallel_level,
+    int style,
+    int dist_metric,
+    bool dist_average,
+    bool single_sig,
     bool progressbar
 );
 
