@@ -6,6 +6,7 @@
 #include <unordered_set>
 #include <unordered_map>
 #include "CppStats.h"
+#include "CppDistances.h"
 #include "spEDMDataStruct.h"
 #include <RcppThread.h>
 
@@ -70,7 +71,7 @@ std::vector<IntersectionRes> IntersectionCardinalitySingle(
             for (size_t iidx = 0; iidx < neighborsX[idx].size(); ++iidx){
               if(lib_set.count(neighborsX[idx][iidx]) > 0){
                 neighbors_x.push_back(neighborsX[idx][iidx]);
-                if (neighbors_x.size() > max_r) break;
+                if (neighbors_x.size() >= max_r) break;
               }
             }
             if (neighbors_x.size() > n_excluded) {
@@ -82,7 +83,7 @@ std::vector<IntersectionRes> IntersectionCardinalitySingle(
             for (size_t iidx = 0; iidx < neighborsY[idx].size(); ++iidx){
               if(lib_set.count(neighborsY[idx][iidx])){
                 neighbors_y.push_back(neighborsY[idx][iidx]);
-                if (neighbors_y.size() > max_r) break;
+                if (neighbors_y.size() >= max_r) break;
               }
             }
             if (neighbors_y.size() > n_excluded) {
@@ -145,7 +146,7 @@ std::vector<IntersectionRes> IntersectionCardinalitySingle(
             for (size_t iidx = 0; iidx < neighborsX[idx].size(); ++iidx){
               if(lib_set.count(neighborsX[idx][iidx]) > 0){
                 neighbors_x.push_back(neighborsX[idx][iidx]);
-                if (neighbors_x.size() > max_r) break;
+                if (neighbors_x.size() >= max_r) break;
               }
             }
             if (neighbors_x.size() > n_excluded) {
@@ -157,7 +158,7 @@ std::vector<IntersectionRes> IntersectionCardinalitySingle(
             for (size_t iidx = 0; iidx < neighborsY[idx].size(); ++iidx){
               if(lib_set.count(neighborsY[idx][iidx])){
                 neighbors_y.push_back(neighborsY[idx][iidx]);
-                if (neighbors_y.size() > max_r) break;
+                if (neighbors_y.size() >= max_r) break;
               }
             }
             if (neighbors_y.size() > n_excluded) {
@@ -309,6 +310,7 @@ std::vector<IntersectionRes> IntersectionCardinalitySingle(
  * @param pred            Vector of prediction indices (shouble be 0-based in C++).
  * @param num_neighbors   Maximum number of neighbors to consider in intersection (e.g., from 1 to k).
  * @param n_excluded      Number of nearest neighbors to exclude (e.g., due to temporal proximity).
+ * @param dist_metric     Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param threads         Number of threads used for parallel computation.
  * @param parallel_level  Parallel mode flag: 0 = parallel, 1 = serial.
  *
@@ -330,7 +332,8 @@ std::vector<double> IntersectionCardinality(
     const std::vector<size_t>& pred,
     size_t num_neighbors,
     size_t n_excluded,
-    int threads,
+    int dist_metric = 2,
+    int threads = 8,
     int parallel_level = 0) {
   std::vector<double> result (num_neighbors, std::numeric_limits<double>::quiet_NaN());
   // Input validation
@@ -355,9 +358,16 @@ std::vector<double> IntersectionCardinality(
   size_t threads_sizet = static_cast<size_t>(std::abs(threads));
   threads_sizet = std::min(static_cast<size_t>(std::thread::hardware_concurrency()), threads_sizet);
 
-  // Precompute neighbors
-  auto nx = CppDistSortedIndice(CppMatDistance(embedding_x, false, true),lib);
-  auto ny = CppDistSortedIndice(CppMatDistance(embedding_y, false, true),lib);
+  // Use L1 norm (Manhattan distance) if dist_metric == 1, else use L2 norm
+  bool L1norm = (dist_metric == 1);
+
+  // // Precompute neighbors (The earlier implementation based on a serial version)
+  // auto nx = CppDistSortedIndice(CppMatDistance(embedding_x, L1norm, true), lib, num_neighbors + n_excluded);
+  // auto ny = CppDistSortedIndice(CppMatDistance(embedding_y, L1norm, true), lib, num_neighbors + n_excluded);
+
+  // Precompute neighbors (parallel computation)
+  auto nx = CppMatKNNeighbors(embedding_x, lib, num_neighbors + n_excluded, threads_sizet, L1norm);
+  auto ny = CppMatKNNeighbors(embedding_y, lib, num_neighbors + n_excluded, threads_sizet, L1norm);
 
   // run cross mapping
   std::vector<IntersectionRes> res = IntersectionCardinalitySingle(
@@ -384,6 +394,7 @@ std::vector<double> IntersectionCardinality(
  *   pred           - Prediction index vector (shouble be 0-based in C++).
  *   num_neighbors  - Number of neighbors used for cross mapping (after exclusion).
  *   n_excluded     - Number of nearest neighbors to exclude (e.g. temporal).
+ *   dist_metric    - Distance metric selector (1: Manhattan, 2: Euclidean).
  *   threads        - Number of threads used in parallel computation.
  *   parallel_level - Whether to use multithreaded (0) or serial (1) mode
  *
@@ -401,7 +412,8 @@ std::vector<double> IntersectionCardinalityScores(
     const std::vector<size_t>& pred,
     size_t num_neighbors,
     size_t n_excluded,
-    int threads,
+    int dist_metric = 2,
+    int threads = 8,
     int parallel_level = 0) {
   // Input validation
   if (embedding_x.size() != embedding_y.size() || embedding_x.empty()) {
@@ -425,9 +437,16 @@ std::vector<double> IntersectionCardinalityScores(
   size_t threads_sizet = static_cast<size_t>(std::abs(threads));
   threads_sizet = std::min(static_cast<size_t>(std::thread::hardware_concurrency()), threads_sizet);
 
-  // Precompute neighbors
-  auto nx = CppDistSortedIndice(CppMatDistance(embedding_x, false, true),lib);
-  auto ny = CppDistSortedIndice(CppMatDistance(embedding_y, false, true),lib);
+  // Use L1 norm (Manhattan distance) if dist_metric == 1, else use L2 norm
+  bool L1norm = (dist_metric == 1);
+
+  // // Precompute neighbors (The earlier implementation based on a serial version)
+  // auto nx = CppDistSortedIndice(CppMatDistance(embedding_x, L1norm, true), lib, num_neighbors + n_excluded);
+  // auto ny = CppDistSortedIndice(CppMatDistance(embedding_y, L1norm, true), lib, num_neighbors + n_excluded);
+
+  // Precompute neighbors (parallel computation)
+  auto nx = CppMatKNNeighbors(embedding_x, lib, num_neighbors + n_excluded, threads_sizet, L1norm);
+  auto ny = CppMatKNNeighbors(embedding_y, lib, num_neighbors + n_excluded, threads_sizet, L1norm);
 
   // run cross mapping
   std::vector<IntersectionRes> res = IntersectionCardinalitySingle(
