@@ -33,6 +33,9 @@
  * @param num_neighbors: Vector specifying the numbers of neighbors to use for Simplex Projection.
  * @param nrow: Number of rows in the input spatial grid data.
  * @param cumulate: Boolean flag to determine whether to cumulate the partial correlations.
+ * @param style: Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average: Whether to average distance by the number of valid vector components.
  * @return A vector of size 2 containing:
  *         - rho[0]: Pearson correlation between the target and its predicted values.
  *         - rho[1]: Partial correlation between the target and its predicted values, adjusting for control variables.
@@ -47,7 +50,10 @@ std::vector<double> PartialSimplex4Grid(
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
     int nrow,
-    bool cumulate
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
 );
 
 /**
@@ -69,6 +75,9 @@ std::vector<double> PartialSimplex4Grid(
  * @param nrow: Number of rows in the input spatial grid data.
  * @param theta: Weighting parameter for distances in the S-Map method.
  * @param cumulate: Boolean flag to determine whether to cumulate the partial correlations.
+ * @param style: Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average: Whether to average distance by the number of valid vector components.
  * @return A vector of size 2 containing:
  *         - rho[0]: Pearson correlation between the target and its predicted values.
  *         - rho[1]: Partial correlation between the target and its predicted values, adjusting for control variables.
@@ -83,8 +92,11 @@ std::vector<double> PartialSMap4Grid(
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
     int nrow,
-    double theta,
-    bool cumulate
+    double theta = 1.0,
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
 );
 
 /**
@@ -110,6 +122,9 @@ std::vector<double> PartialSMap4Grid(
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * @param cumulate             Whether to cumulate the partial correlations.
  * @param row_size_mark        If ture, use the row-wise libsize to mark the libsize; if false, use col-wise libsize.
+ * @param style                Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average         Whether to average distance by the number of valid vector components.
  *
  * @return  A vector contains the library size and the corresponding cross mapping and partial cross mapping result.
  */
@@ -130,7 +145,10 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
     size_t threads,
     int parallel_level,
     bool cumulate,
-    bool row_size_mark
+    bool row_size_mark,
+    int style,
+    int dist_metric,
+    bool dist_average
 );
 
 /**
@@ -155,6 +173,9 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
  * @param threads              The number of threads to use for parallel processing
  * @param parallel_level       Level of parallel computing: 0 for `lower`, 1 for `higher`
  * @param cumulate             Enable cumulative partial correlations
+ * @param style                Embedding style selector (0: includes current state, 1: excludes it)
+ * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean)
+ * @param dist_average         Whether to average distance by the number of valid vector components
  *
  * @return Vector of PartialCorRes containing mapping results for each library configuration
  */
@@ -174,7 +195,10 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
     double theta,
     size_t threads,
     int parallel_level,
-    bool cumulate
+    bool cumulate,
+    int style,
+    int dist_metric,
+    bool dist_average
 );
 
 /**
@@ -199,6 +223,10 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
  * - threads: Number of threads to use for parallel computation.
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
+ * - style: Embedding style selector (0: includes current state, 1: excludes it).
+ * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Boolean flag indicating whether to display a progress bar during computation.
  *
  * Returns:
@@ -228,6 +256,10 @@ std::vector<std::vector<double>> SCPCM4Grid(
     int threads,                                         // Number of threads used from the global pool
     int parallel_level,                                  // Level of parallel computing: 0 for `lower`, 1 for `higher`
     bool cumulate,                                       // Whether to cumulate the partial correlations
+    int style,                                           // Embedding style selector (0: includes current state, 1: excludes it)
+    int dist_metric,                                     // Distance metric selector (1: Manhattan, 2: Euclidean)
+    bool dist_average,                                   // Whether to average distance by the number of valid vector components
+    bool single_sig,                                     // Whether to estimate significance and confidence intervals using a single rho value
     bool progressbar                                     // Whether to print the progress bar
 );
 
@@ -249,6 +281,10 @@ std::vector<std::vector<double>> SCPCM4Grid(
  * - threads: Number of parallel computation threads
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - cumulate: Enable cumulative partial correlations
+ * - style: Embedding style selector (0: includes current state, 1: excludes it).
+ * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Display progress bar during computation
  *
  * Returns:
@@ -278,6 +314,10 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
     int threads,
     int parallel_level,
     bool cumulate,
+    int style,
+    int dist_metric,
+    bool dist_average,
+    bool single_sig,
     bool progressbar
 );
 
