@@ -28,6 +28,9 @@
  * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
  * @param num_neighbors: Vector specifying the numbers of neighbors to use for simplex projection.
  * @param cumulate: Flag indicating whether to cumulatively incorporate control variables.
+ * @param style: Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average: Whether to average distance by the number of valid vector components.
  *
  * @return A std::vector<double> containing:
  *         - rho[0]: Pearson correlation between the target and its simplex projection.
@@ -43,7 +46,10 @@ std::vector<double> PartialSimplex4Lattice(
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
-    bool cumulate
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
 ){
   int n_controls = controls.size();
   std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
@@ -54,15 +60,15 @@ std::vector<double> PartialSimplex4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       if (i == 0){
-        temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0]);
+        temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], dist_metric, dist_average);
       } else {
-        temp_pred = SimplexProjectionPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i]);
+        temp_pred = SimplexProjectionPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i], dist_metric, dist_average);
       }
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i],style);
     }
 
-    std::vector<double> con_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls]);
-    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0]);
+    std::vector<double> con_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls], dist_metric, dist_average);
+    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], dist_metric, dist_average);
 
     if (checkOneDimVectorNotNanNum(target_pred) >= 3){
       rho[0] = PearsonCor(target,target_pred,true);
@@ -74,12 +80,12 @@ std::vector<double> PartialSimplex4Lattice(
     std::vector<std::vector<double>> temp_embedding;
 
     for (int i = 0; i < n_controls; ++i) {
-      temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0]);
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
-      temp_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1]);
+      temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], dist_metric, dist_average);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i],style);
+      temp_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1], dist_metric, dist_average);
       con_pred[i] = temp_pred;
     }
-    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0]);
+    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], dist_metric, dist_average);
 
     if (checkOneDimVectorNotNanNum(target_pred) >= 3){
       rho[0] = PearsonCor(target,target_pred,true);
@@ -109,6 +115,9 @@ std::vector<double> PartialSimplex4Lattice(
  * @param num_neighbors: Vector specifying the numbers of neighbors to use for S-Map prediction.
  * @param theta: Weighting parameter for distances in S-Map.
  * @param cumulate: Boolean flag to determine whether to cumulate the partial correlations.
+ * @param style: Embedding style selector (0: includes current state, 1: excludes it).
+ * @param dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * @param dist_average: Whether to average distance by the number of valid vector components.
  * @return A vector of size 2 containing:
  *         - rho[0]: Pearson correlation between the target and its predicted values.
  *         - rho[1]: Partial correlation between the target and its predicted values, adjusting for control variables.
@@ -123,8 +132,11 @@ std::vector<double> PartialSMap4Lattice(
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
-    double theta,
-    bool cumulate
+    double theta = 1.0,
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
 ){
   int n_controls = controls.size();
   std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
@@ -135,15 +147,15 @@ std::vector<double> PartialSMap4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       if (i == 0){
-        temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta);
+        temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta, dist_metric, dist_average);
       } else {
-        temp_pred = SMapPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i], theta);
+        temp_pred = SMapPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i], theta, dist_metric, dist_average);
       }
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i],style);
     }
 
-    std::vector<double> con_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls], theta);
-    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta);
+    std::vector<double> con_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls], theta, dist_metric, dist_average);
+    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta, dist_metric, dist_average);
 
     if (checkOneDimVectorNotNanNum(target_pred) >= 3){
       rho[0] = PearsonCor(target,target_pred,true);
@@ -155,12 +167,12 @@ std::vector<double> PartialSMap4Lattice(
     std::vector<std::vector<double>> temp_embedding;
 
     for (int i = 0; i < n_controls; ++i) {
-      temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta);
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
-      temp_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1], theta);
+      temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta, dist_metric, dist_average);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i],style);
+      temp_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1], theta, dist_metric, dist_average);
       con_pred[i] = temp_pred;
     }
-    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta);
+    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta, dist_metric, dist_average);
 
     if (checkOneDimVectorNotNanNum(target_pred) >= 3){
       rho[0] = PearsonCor(target,target_pred,true);
@@ -190,6 +202,9 @@ std::vector<double> PartialSMap4Lattice(
  *   - threads: The number of threads to use for parallel processing.
  *   - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  *   - cumulate: Whether to accumulate partial correlations.
+ *   - style: Embedding style selector (0: includes current state, 1: excludes it).
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components.
  *
  * Returns:
  *   A vector of PartialCorRes objects, where each contains:
@@ -212,7 +227,10 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
     double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
     size_t threads,                                     // Number of threads to use for parallel processing
     int parallel_level,                                 // Level of parallel computing: 0 for `lower`, 1 for `higher`
-    bool cumulate                                       // Whether to cumulate the partial correlations
+    bool cumulate,                                      // Whether to cumulate the partial correlations
+    int style,                                          // Embedding style selector (0: includes current state, 1: excludes it)
+    int dist_metric,                                    // Distance metric selector (1: Manhattan, 2: Euclidean)
+    bool dist_average                                   // Whether to average distance by the number of valid vector components
 ) {
   int max_lib_size = lib_indices.size();
 
@@ -222,9 +240,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
     // Run partial cross map and store results
     std::vector<double> rho;
     if (simplex) {
-      rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
+      rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate, style, dist_metric, dist_average);
     } else {
-      rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
+      rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate, style, dist_metric, dist_average);
     }
     x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
     return x_xmap_y;
@@ -258,9 +276,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
       // Run partial cross map and store results
       std::vector<double> rho;
       if (simplex) {
-        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, valid_lib_indices[i], pred_indices, conEs, taus, b, cumulate);
+        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, valid_lib_indices[i], pred_indices, conEs, taus, b, cumulate, style, dist_metric, dist_average);
       } else {
-        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, valid_lib_indices[i], pred_indices, conEs, taus, b, theta, cumulate);
+        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, valid_lib_indices[i], pred_indices, conEs, taus, b, theta, cumulate, style, dist_metric, dist_average);
       }
       // Directly initialize a PartialCorRes struct with the three values
       PartialCorRes result(lib_size, rho[0], rho[1]);
@@ -291,9 +309,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
       // Run partial cross map and store results
       std::vector<double> rho;
       if (simplex) {
-        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, local_lib_indices, pred_indices, conEs, taus, b, cumulate);
+        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, local_lib_indices, pred_indices, conEs, taus, b, cumulate, style, dist_metric, dist_average);
       } else {
-        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, local_lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
+        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, local_lib_indices, pred_indices, conEs, taus, b, theta, cumulate, style, dist_metric, dist_average);
       }
       x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
     }
@@ -319,8 +337,12 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
  * - simplex: Boolean flag indicating whether to use simplex projection (true) or S-mapping (false) for prediction.
  * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
  * - threads: Number of threads to use for parallel computation.
- * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
+ * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
+ * - style: Embedding style selector (0: includes current state, 1: excludes it).
+ * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Boolean flag indicating whether to display a progress bar during computation.
  *
  * Returns:
@@ -351,6 +373,10 @@ std::vector<std::vector<double>> SCPCM4Lattice(
     int threads,                                        // Number of threads used from the global pool
     int parallel_level,                                 // Level of parallel computing: 0 for `lower`, 1 for `higher`
     bool cumulate,                                      // Whether to cumulate the partial correlations
+    int style,                                          // Embedding style selector (0: includes current state, 1: excludes it)
+    int dist_metric,                                    // Distance metric selector (1: Manhattan, 2: Euclidean)
+    bool dist_average,                                  // Whether to average distance by the number of valid vector components
+    bool single_sig,                                    // Whether to estimate significance and confidence intervals using a single rho value
     bool progressbar                                    // Whether to print the progress bar
 ) {
   // If b is not provided correctly, default it to E + 2
@@ -373,7 +399,7 @@ std::vector<std::vector<double>> SCPCM4Lattice(
   size_t threads_sizet = static_cast<size_t>(std::abs(threads));
   threads_sizet = std::min(static_cast<size_t>(std::thread::hardware_concurrency()), threads_sizet);
 
-  std::vector<std::vector<double>> x_vectors = GenLatticeEmbeddings(x,nb_vec,Ex,taux);
+  std::vector<std::vector<double>> x_vectors = GenLatticeEmbeddings(x,nb_vec,Ex,taux,style);
   size_t n = pred.size();
 
   size_t n_confounds;
@@ -422,7 +448,10 @@ std::vector<std::vector<double>> SCPCM4Lattice(
           theta,
           threads_sizet,
           parallel_level,
-          cumulate
+          cumulate,
+          style,
+          dist_metric,
+          dist_average
         );
         bar++;
       }
@@ -443,7 +472,10 @@ std::vector<std::vector<double>> SCPCM4Lattice(
           theta,
           threads_sizet,
           parallel_level,
-          cumulate
+          cumulate,
+          style,
+          dist_metric,
+          dist_average
         );
       }
     }
@@ -468,7 +500,10 @@ std::vector<std::vector<double>> SCPCM4Lattice(
           theta,
           threads_sizet,
           parallel_level,
-          cumulate
+          cumulate,
+          style,
+          dist_metric,
+          dist_average
         );
         bar++;
       }, threads_sizet);
@@ -490,7 +525,10 @@ std::vector<std::vector<double>> SCPCM4Lattice(
           theta,
           threads_sizet,
           parallel_level,
-          cumulate
+          cumulate,
+          style,
+          dist_metric,
+          dist_average
         );
       }, threads_sizet);
     }
@@ -513,42 +551,80 @@ std::vector<std::vector<double>> SCPCM4Lattice(
 
   std::vector<std::vector<double>> final_results;
 
-  // Compute the mean of second and third values for each group
-  for (const auto& group : grouped_results) {
-    std::vector<double> second_values, third_values;
+  if (single_sig) {
+    // Calculate significance and confidence intervals using the mean of rho vector only.
+    for (const auto& group : grouped_results) {
+      // Compute the mean of second and third values for each group
+      std::vector<double> second_values, third_values;
 
-    for (const auto& val : group.second) {
-      second_values.push_back(val.first);
-      third_values.push_back(val.second);
+      for (const auto& val : group.second) {
+        second_values.push_back(val.first);
+        third_values.push_back(val.second);
+      }
+
+      double mean_second = CppMean(second_values, true);
+      double mean_third = CppMean(third_values, true);
+
+      final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
     }
 
-    double mean_second = CppMean(second_values, true);
-    double mean_third = CppMean(third_values, true);
+    // Compute significance and confidence intervals for each result
+    for (size_t i = 0; i < final_results.size(); ++i) {
+      double rho_second = final_results[i][1];
+      double rho_third = final_results[i][2];
 
-    final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
-  }
+      // Compute significance and confidence interval for second value
+      double significance_second = CppCorSignificance(rho_second, n);
+      std::vector<double> confidence_interval_second = CppCorConfidence(rho_second, n);
 
-  // Compute significance and confidence intervals for each result
-  for (size_t i = 0; i < final_results.size(); ++i) {
-    double rho_second = final_results[i][1];
-    double rho_third = final_results[i][2];
+      // Compute significance and confidence interval for third value
+      double significance_third = CppCorSignificance(rho_third, n, n_confounds);
+      std::vector<double> confidence_interval_third = CppCorConfidence(rho_third, n, n_confounds);
 
-    // Compute significance and confidence interval for second value
-    double significance_second = CppCorSignificance(rho_second, n);
-    std::vector<double> confidence_interval_second = CppCorConfidence(rho_second, n);
+      // Append computed statistical values to the result
+      final_results[i].push_back(significance_second);
+      final_results[i].push_back(confidence_interval_second[0]);
+      final_results[i].push_back(confidence_interval_second[1]);
 
-    // Compute significance and confidence interval for third value
-    double significance_third = CppCorSignificance(rho_third, n, n_confounds);
-    std::vector<double> confidence_interval_third = CppCorConfidence(rho_third, n, n_confounds);
+      final_results[i].push_back(significance_third);
+      final_results[i].push_back(confidence_interval_third[0]);
+      final_results[i].push_back(confidence_interval_third[1]);
+    }
+  } else {
+    // For each group, compute the mean of second and third values and calculate significance and confidence intervals using the original vectors
+    for (const auto& group : grouped_results) {
+      std::vector<double> second_values, third_values;
 
-    // Append computed statistical values to the result
-    final_results[i].push_back(significance_second);
-    final_results[i].push_back(confidence_interval_second[0]);
-    final_results[i].push_back(confidence_interval_second[1]);
+      // Collect all second and third values from current group
+      for (const auto& val : group.second) {
+        second_values.push_back(val.first);
+        third_values.push_back(val.second);
+      }
 
-    final_results[i].push_back(significance_third);
-    final_results[i].push_back(confidence_interval_third[0]);
-    final_results[i].push_back(confidence_interval_third[1]);
+      // Compute mean values for reporting
+      double mean_second = CppMean(second_values, true);
+      double mean_third = CppMean(third_values, true);
+
+      // Compute significance and confidence intervals using the full vector of values (not just mean)
+      double significance_second = CppMeanCorSignificance(second_values, n);
+      std::vector<double> confidence_interval_second = CppMeanCorConfidence(second_values, n);
+
+      double significance_third = CppMeanCorSignificance(third_values, n, n_confounds);
+      std::vector<double> confidence_interval_third = CppMeanCorConfidence(third_values, n, n_confounds);
+
+      // Store group ID, mean values, and corresponding statistical results
+      final_results.push_back({
+        static_cast<double>(group.first),
+        mean_second,
+        mean_third,
+        significance_second,
+        confidence_interval_second[0],
+        confidence_interval_second[1],
+        significance_third,
+        confidence_interval_third[0],
+        confidence_interval_third[1]
+      });
+    }
   }
 
   return final_results;
