@@ -10,24 +10,41 @@
 #include "CppStats.h"
 
 /*
- * Computes predictions using the simplex projection method based on state-space reconstruction.
+ * Perform simplex projection prediction using state-space reconstruction.
+ *
+ * Given reconstructed state-space vectors and corresponding target values,
+ * this function predicts target values at specified prediction indices by
+ * weighting nearest neighbors from a given library set.
+ *
+ * Distance calculations exclude NaN components to ensure numerical stability.
+ * Supports two distance metrics and optional averaging of distance by the
+ * number of valid vector components.
+ *
+ * Supported distance metrics:
+ *   dist_metric = 1: L1 (Manhattan) distance
+ *   dist_metric = 2: L2 (Euclidean) distance
  *
  * Parameters:
- *   - vectors: Reconstructed state-space (each row represents a separate vector/state).
- *   - target: Spatial cross sectional series used as the target (should align with vectors).
- *   - lib_indices: Vector of indices indicating which states to include when searching for neighbors.
- *   - pred_indices: Vector of indices indicating which states to predict from.
- *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *   vectors        - 2D vector of reconstructed state-space vectors; vectors[i] corresponds to spatial unit i.
+ *   target         - Target values for each spatial unit.
+ *   lib_indices    - Indices specifying states used as the library (neighbors).
+ *   pred_indices   - Indices specifying states to predict.
+ *   num_neighbors  - Number of nearest neighbors considered for prediction. Default is 4.
+ *   dist_metric    - Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   dist_average   - Whether to average distance by the number of valid vector components. Default is true.
  *
  * Returns:
- *   A vector<double> containing predicted target values (target_pred).
+ *   A vector<double> of predicted target values aligned with input target size.
+ *   Entries are NaN if prediction is not possible for that index.
  */
 std::vector<double> SimplexProjectionPrediction(
     const std::vector<std::vector<double>>& vectors,
     const std::vector<double>& target,
     const std::vector<int>& lib_indices,
     const std::vector<int>& pred_indices,
-    int num_neighbors
+    int num_neighbors = 4,
+    int dist_metric = 2,
+    bool dist_average = true
 );
 
 /*
@@ -38,7 +55,9 @@ std::vector<double> SimplexProjectionPrediction(
  *   - target: Spatial cross sectional series used as the target (should align with vectors).
  *   - lib_indices: Vector of indices indicating which states to include when searching for neighbors.
  *   - pred_indices: Vector of indices indicating which states to use for prediction.
- *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *   - num_neighbors: Number of neighbors to use for simplex projection. Default is 4.
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
  *
  * Returns:
  *   A double representing the Pearson correlation coefficient (rho) between the predicted and actual target values.
@@ -48,7 +67,9 @@ double SimplexProjection(
     const std::vector<double>& target,
     const std::vector<int>& lib_indices,
     const std::vector<int>& pred_indices,
-    int num_neighbors
+    int num_neighbors = 4,
+    int dist_metric = 2,
+    bool dist_average = true
 );
 
 /*
@@ -59,7 +80,9 @@ double SimplexProjection(
  *   - target: Spatial cross sectional series to be used as the target (should align with vectors).
  *   - lib_indices: Vector of indices indicating which states to include when searching for neighbors.
  *   - pred_indices: Vector of indices indicating which states to predict from.
- *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *   - num_neighbors: Number of neighbors to use for simplex projection. Default is 4.
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). Default is 2 (Euclidean).
+ *   - dist_average: Whether to average distance by the number of valid vector components. Default is true.
  *
  * Returns:
  *   A vector<double> containing:
@@ -72,7 +95,9 @@ std::vector<double> SimplexBehavior(
     const std::vector<double>& target,
     const std::vector<int>& lib_indices,
     const std::vector<int>& pred_indices,
-    int num_neighbors
+    int num_neighbors = 4,
+    int dist_metric = 2,
+    bool dist_average = true
 );
 
 #endif // SimplexProjection_H
